@@ -3,11 +3,11 @@
 mod opcode;
 pub use opcode::*;
 
-mod rclass;
-pub use rclass::*;
+mod class;
+pub use class::*;
 
-mod rtype;
-pub use rtype::*;
+mod r#type;
+pub use r#type::*;
 
 mod rcode;
 pub use rcode::*;
