@@ -1,4 +1,9 @@
-use crate::{constants::DOMAIN_NAME_MAX_LENGTH, names::Name, Error, Result};
+use crate::{
+    bytes::{Cursor, Reader},
+    constants::DOMAIN_NAME_MAX_LENGTH,
+    names::Name,
+    Error, Result,
+};
 use arrayvec::ArrayString;
 use std::{
     cmp::Ordering,
@@ -428,6 +433,11 @@ impl super::private::DNameBase for InlineName {
     fn set_root(&mut self) {
         self.set_root()
     }
+
+    #[inline(always)]
+    fn from_cursor(c: &mut Cursor<'_>) -> Result<Self> {
+        c.read()
+    }
 }
 
 impl super::DName for InlineName {}
