@@ -1,7 +1,7 @@
 use crate::{
-    constants::{RClass, RType},
+    constants::{Class, Type},
+    names::InlineName,
     records::data::RecordData,
-    InlineName,
 };
 
 /// A resource record.
@@ -11,10 +11,10 @@ pub struct ResourceRecord {
     pub name: InlineName,
 
     /// The record class.
-    pub rclass: RClass,
+    pub rclass: Class,
 
     /// The record type.
-    pub rtype: RType,
+    pub rtype: Type,
 
     /// The time (in seconds) that the resource record may be cached before it should
     /// be discarded. Zero value means the record should not be cached.
